/*   
  Sketch for testing change blindness response times
  Original by dr. Maarten Wijntjes (M.W.A.Wijntjes@tudelft.nl)
 
  Additions by Bart Kallenbach (bart@kallenba.ch): 
    CSV instead of tab delimited txt files (Opens in excel)
    Added support for multiple sets of images
    Added pause screens between these images
    Fixed timing issues
*/

//These two strings determine the filename of the datafile
String description = "redesign";
String subjectName = "Piet";

//This is a collection of the images you want to test.
//For each set of images you want to test, add a line that looks like this:
// {"filename1", "filename2", "description of element"},
// Include the comma for each line.
String[][] images = {
                      {"with-01.jpg", "without-01.jpg", "What is fab"}, 
                      {"with-01.jpg", "without-02.jpg", "Section headings"},
                      {"with-01.jpg", "without-03.jpg", "Featured"},
                      {"with-01.jpg", "without-04.jpg", "Categories menu"},
                      {"with-01.jpg", "without-05.jpg", "Search bar"},
                      {"with-02.jpg", "without-06.jpg", "Footer menu"},
                    };


//set the sketch to open in fullscreen
//remove the function to disable this.
boolean sketchFullScreen() {
  return true;
}


//You don't need to edit anything below this line. (Unless you're not afraid of errors)
                    
float fractionImageBlank=.2;  //the fraction of time that the image will be blank
int nFrame=30;                //framerate
int nOff=round(fractionImageBlank*nFrame/2); 
int nOn=nFrame/2-nOff;
PrintWriter output;
PImage with,without;
int iImage = 0;
int lastTime = 0;
int thisTime = 0;
boolean paused = true;


//handle user clicks
void clickHandler() {
            //Get current clock
            thisTime = millis();
    
            //Add a line to the file
            output.println(images[iImage][2] + "," + mouseX + "," + mouseY + "," + (thisTime - lastTime));
                        
            //if we still have images left, load a new one
            if (iImage < (images.length - 1)) {
              iImage++;
              with=loadImage(images[iImage][0]);
              without=loadImage(images[iImage][1]);
              paused = true;              
            } else {     
             //if not, close the file and exit the sketch.
              output.flush(); 
              output.close();
              exit();            
            }
        
}

void setup() {
  frameRate(nFrame);
  with=loadImage(images[0][0]);
  without=loadImage(images[0][1]);
  size(with.width, with.height);             //size of screen is set equal to size of images
  smooth();                                    //why not... or is this computationally heavy???
  output = createWriter("results/" + description + "-" + subjectName + "-data.csv"); 
  output.println("Element, Mouse X, MouseY, time");
  ellipseMode(CENTER);
  noCursor();
 
}

void draw() {
  if (paused) {
                 background(128);
                 text("Press the spacebar to continue", (with.width/2-100), (with.height/2));
  } else {
          //give the time in the title:
          //frame.setTitle("time " + int(round(millis())));
          
          // Calculate the frame to display, use % to cycle through frames 
          int frame = frameCount % nFrame;
          // 4 if-statements that decide which image is shown
          if (0<frame&&frame<nOn){image(with,0,0);}
          if (nOn<frame&&frame<nFrame/2){background(128);}
          if (nFrame/2<frame&&frame<nFrame/2+nOn){image(without,0,0);}
          if (nFrame/2+nOn<frame&&frame<nFrame){background(128);}
          
          // The mouse is cursor is a gray circle, which is drawn here
          noFill();
          strokeWeight(4);
          stroke(128);
          // By using this if-statement the circle is not shown at the start (see what happens if you remove it)
          if (mouseX!=0&&mouseY!=0){
          ellipse(mouseX,mouseY,50,50);// mouse selection ellips
          }

  }
  // If the mouse is pressed, the location of the mouse is written 
  // to a data file and the next set of images is loaded.
  // This should only work when we're not at a pause screen.
  if (mousePressed && paused == false) {
   clickHandler();
  }
  
  //if we're paused, unpause the sketch when a key is pressed
  if (keyPressed && paused == true) {
     lastTime = millis();
     paused = false;
  }
  
}
